// *****************************************************************
// ****		          UART0 and other debug routines            ****
// ****                     www.tolaemon.com                    ****
// ****                       Source code                       ****
// ****                       16-10-2008                        ****
// ****                     Jordi Bartolome                     ****
// ****                                                         ****
// **** IMPORTANT:                                              ****
// **** Using parts of this code means accepting all conditions ****
// **** exposed on the web: www.tolaemon.com/hplayer            ****
// *****************************************************************

#include "uart0.h"

volatile t_uart_buffer UART0_buffers;


/*	Description:
	Initializes UART0 hardware and structures
	Recives:
	Returns:
*/
void UART_Init( ){

	// USART is configured
	UBRR0H = 0;// High byte. Datasheet indicates, that to work at XXXXXbps with a system clock of XX-E6 Hz, UBRR must be set to XXX
	UBRR0L = 103;// Low byte. Datasheet indicates, that to work at XXXXXbps with a system clock of XX-E6 Hz, UBRR must be set to XXX
	UCSR0B = (1<<RXCIE)|(1<<RXEN)|(1<<TXEN);/* Enable receiver, transmitter, and reciver interrupts */
	UCSR0C = (1<<UCSZ0)|(1<<UCSZ1);/* Set frame format: 8data,1stop bit */

	//UART0 control vars are Reset
	UART0_Reset( );

};// UART_Init



/*	Description:
	Resets UART0 control variables
	Recives:
	Returns:
*/
void UART0_Reset( ){

	// UART0 buffers reset
	// Reception buffer
	UART0_buffers.recv_write_index=1;
	UART0_buffers.recv_read_index=0;
	// Transmision buffer
	UART0_buffers.trans_write_index=1;
	UART0_buffers.trans_read_index=0;

};//UART0_Reset



/*	Description:
	Sends a byte through UART0 if it is ready and returns an acknowldge, other way it doesn't send anything
 and returns error  
	Recives:
		by value data: int8_t with the byte which is going to be sent
	Returns:
		by value: int8_t with the result of the operation:
			FALSE: UART0 is busy
			TRUE: byte has been sent
 */
uint8_t UART0_TransmitByte( uint8_t data ){
	uint8_t ui8_aux;

	if ( !( UCSR0A & (1<<UDRE)) ){
		// if buffer is busy we don't send anything
		ui8_aux= FALSE;
	}else{
		UDR0 = data;
		ui8_aux= TRUE;
	};//while
	// si el buffer ya esta vacio se pone el dato para q se envie

	return ui8_aux;
	
};//UART0_TransmitByte



/*	Description:
	Forcs a byte to be transmited through UART0
	Recives:
		by value data: int8_t with the byte which is going to be sent
	Returns:
 */
void UART0_ForceTransmitByte( uint8_t data ){

	while ( !( UCSR0A & (1<<UDRE)) ){};
	UDR0 = data;
	
};//UART0_ForceTransmitByte



/*	Description:
	Checks if transmision buffer is empty or not, in other words, if there are bytes
	to send into the transmision buffer
	Recives:
	Returns:
		TRUE: if there are NO bytes to send in the transmision buffer
		FALSE: if it is not empty, there are bytes to send in the transmision buffer
 */
uint8_t UART0_TransmisionBufferEmpty( ){
	uint8_t ui8_aux;

	if ( ((UART0_buffers.trans_read_index+1)%MAX_UART_BUFFER)!=UART0_buffers.trans_write_index){
		ui8_aux=FALSE;
	}else{
		ui8_aux=TRUE;
	};//if

	return ui8_aux;

};//UART0_TransmisionBufferEmpty



/*	Description:
	Polls the UART0 transimsion buffer, looking if there is any other byte to send on it. So if there are
    still bytes to send (and the send register is free) next byte of the buffer is sent.
	Recives:
	Returns:
		TRUE: if a byte has been sent
		FALSE: if no byte has been sent ( because transmission routine is busy, or there are no bytes to transmit)
 */
uint8_t UART0_PollBufferTransmission( ){
	uint8_t ui8_aux;

	// we have to check that buffer is not empty before reading the buffer and send the read byte 
	if ( ((UART0_buffers.trans_read_index+1)%MAX_UART_BUFFER)!=UART0_buffers.trans_write_index){
		// if transmision_buffer is not empty byte is read and sent
		ui8_aux=UART0_buffers.trans_buffer[((UART0_buffers.trans_read_index+1)%MAX_UART_BUFFER) ];
		if (UART0_TransmitByte(ui8_aux)==TRUE){
			UART0_buffers.trans_read_index=(UART0_buffers.trans_read_index+1)% MAX_UART_BUFFER;
			ui8_aux=TRUE;
		}else{
			ui8_aux=FALSE;
		};//if	
	}else{
		ui8_aux=FALSE;
	};//if
	
	return ui8_aux; // the return value is set

};//UART0_PollBufferTransmission



/*	Description:
	Forces to send all the content of buffer transmission
	Recives:
	Returns:
 */
void UART0_ForceBufferTransmission( ){
	
	// the transmision buffer is polled until there are no bytes on it
	while (UART0_TransmisionBufferEmpty()==FALSE){
		UART0_PollBufferTransmission();
	};//while

};//UART0_ForceBufferTransmission



/*	Description:
	Inserts a byte into the UART transmission buffer, in order to send it through UART
	Recives:
		data: with the byte we want to send
	Returns:
		TRUE: if the byte has been pushed into the buffer
		FALSE: if has not been possible to push the byte into the buffer
 */
uint8_t UART0_PushByteIntoTransmissionBuffer( uint8_t data ){
	uint8_t ui8_aux;
	
	// before pushing a byte into the transmision buffer we have to check that still there is space to push it
	if (UART0_buffers.trans_write_index!=UART0_buffers.trans_read_index){
		UART0_buffers.trans_buffer[UART0_buffers.trans_write_index]=data;
		UART0_buffers.trans_write_index=(UART0_buffers.trans_write_index+1)%MAX_UART_BUFFER;
		ui8_aux=TRUE;
	}else{
		ui8_aux=FALSE;
	};//if
	
	return ui8_aux;

};//UART0_PushByteIntoTransmissionBuffer



/*	Description:
	When a byte is recived, interrupt pushes it into reception buffer. 
	Important: .recive_error flag is set to TRUE when:
		- a byte is recived and buffer is full, 
		- a byte is recived but there is already a command in the buffer
		- when last byte in the buffer is recived, there is no a command on it, and it is diferent than END_COMMAND_SYMBOL ( buffer is full of bytes, but they dont compose a command )
	Recives:
	Returns:
 */
ISR(SIG_UART0_RECV){
   uint8_t ui8_aux;
		
	// before pushing a byte into the reception buffer we have to check that still there is space to push it
	if (UART0_buffers.recv_write_index!=UART0_buffers.recv_read_index){
		UART0_buffers.recv_buffer[UART0_buffers.recv_write_index]=UDR0;
		UART0_buffers.recv_write_index=(UART0_buffers.recv_write_index+1)%MAX_UART_BUFFER;
		ui8_aux=TRUE;
	}else{
		ui8_aux=FALSE;
	};//if

}//ISR(SIG_OVERFLOW0)



/*	Description:
	 Checks if there are bytes waiting in the reception buffer, and returns TRUE and the 
	 byte value if there are bytes. Returns FALSE if the reception buffer is empty.
	Recives:
	Returns:
	 By value:
		TRUE: if a byte has been read from reception buffer
		FALSE: if no byte has been read
	 By reference:
		data: a char with the read value
 */
uint8_t UART0_ReadRecivedByte(uint8_t * data ){
	uint8_t ui8_aux;
	
	// before reading the buffer we have to check that buffer is not empty
	if ( ((UART0_buffers.recv_read_index+1)%MAX_UART_BUFFER)!=UART0_buffers.recv_write_index){
		// if reception_buffer is not empty byte is read
		*data=UART0_buffers.recv_buffer[((UART0_buffers.recv_read_index+1)%MAX_UART_BUFFER) ];
		UART0_buffers.recv_read_index=(UART0_buffers.recv_read_index+1)% MAX_UART_BUFFER;
		ui8_aux=TRUE;
	}else{
		ui8_aux=FALSE;
	};//if
	
	return ui8_aux; // the return value is set
	
};//UART0_ParseRecivedByte



/*	Description:
		Prints the text representation of the number through the serial Port
	Recives:
		by value: value uint32_t with the unsigned integer we want to print
	Returns:
		by value: 
			FALSE: if has been impossible to set the number into the transmision buffer
			TRUE: if the number representation has been placed into the transision buffer
 */
uint8_t UART0_PrintUint( uint32_t value ){
	uint8_t ascii[10];
	int8_t i,return_value;
	
	// first we get and store all digits into ascii array
	i=0;
	return_value=TRUE;
	while (return_value==TRUE){
		ascii[i]=48+(value%10);
		value=(uint32_t)value/10;// decimal part is omitted
		i++;
		if (value==0) return_value=FALSE;
	};//
	
	// when all ASCII digits have been calculated they are pushed into transmision buffer in the right order
	return_value=TRUE;
	while ((return_value==TRUE)&&(i>0)){
		i--;
		return_value=UART0_PushByteIntoTransmissionBuffer( ascii[i] );
	};//while
	
	return return_value;

};//UART0_PrintUint



/*	Description:
		Prints the text representation of the number into the recived string
	Recives:
		by value: value uint32_t with the unsigned integer we want to set into the string
		by reference: string uint8_t pointer to the string where we want to insert the unisgned integer
		!! BE SURE THAT THE STRING IS BIG ENOUGH TO PLACE THE VALUE REPRESENTATION, OTHERWAY IT WILL CAUSE SYSTEM FAIL DUE TO MEMORY OVERWRITE
	Returns:
		by refernce: 
			the string with the text representation of the unsigned int, if no errors have ocurred
 */
void UART0_UintToString( uint32_t value, uint8_t * string ){
	uint8_t ascii[10];
	int8_t i,j,exit;
	
	// first we get and store all digits into ascii array
	i=0;
	exit=FALSE;
	while (exit==FALSE){
		ascii[i]=48+(value%10);
		value=(uint32_t)value/10;// decimal part is omitted
		i++;
		if (value==0) exit=TRUE;
	};//
	
	// when all ASCII digits have been calculated they are placed into the target string
	j=0;
	while (i>0){
		i--;
		string[j]=ascii[i];
		j++;
	};//while
	string[j]='\0';
	
};//UART0_UintToString



/*	Description:
		Prints the string through serial port
	Recives:
		by ref: uint8_t * to the string we want to print through serial port
	Returns:
		by value: 
			FALSE: if has been impossible to send the string through the serial prot
			TRUE: if the string has been sent through the serial port without problems
 */
uint8_t UART0_PrintString( uint8_t * string){
	int8_t i;
	uint8_t return_value;
	
	i=0;
	return_value=TRUE;
	while ((string[i]!='\0')&&(return_value==TRUE)){
		return_value=UART0_PushByteIntoTransmissionBuffer((uint8_t) string[i] );
		i++;
	};//while
	
	return return_value;

};//UART0_PrintString



/*	Description:
		Prints a char or byte into the buffer
	Recives:
		by valyue: the asccii code or byte we want to print
	Returns:
		by value: 
			FALSE: if has been impossible to send the string through the serial prot
			TRUE: if the string has been sent through the serial port without problems
 */
uint8_t UART0_PrintByte( uint8_t byte_to_print){
	uint8_t return_value;
	
	return_value=UART0_PushByteIntoTransmissionBuffer(byte_to_print);
	
	return return_value;

};//UART0_PrintByte



/*	Description:
	Reads bytes from the reception buffer and executes an action depending on read value
	Recives:
	Returns:
	 By value:
	 By reference:
 */
void UART0_ParseRecivedBytes(){
	uint8_t data;

	if (UART0_ReadRecivedByte(&data)==TRUE){

		switch (data){
			case '0':
				UART0_PrintString("Ini:");
				if (FAT32_Initialize()>0){
					UART0_PrintString("Ok");
				}else{
					UART0_PrintString("Ko");
				};//if
				UART0_PushByteIntoTransmissionBuffer(LINE_FEED);
				UART0_PushByteIntoTransmissionBuffer(CARRIAGE_RETURN);
				break;
			case '1':
				FAT32_Test(2);
				UART0_PushByteIntoTransmissionBuffer(LINE_FEED);
				UART0_PushByteIntoTransmissionBuffer(CARRIAGE_RETURN);
				break;
			case '2':
				FAT32_Test(3);
				UART0_PushByteIntoTransmissionBuffer(LINE_FEED);
				UART0_PushByteIntoTransmissionBuffer(CARRIAGE_RETURN);
				break;
			case '3':
				FAT32_Test(4);
				UART0_PushByteIntoTransmissionBuffer(LINE_FEED);
				UART0_PushByteIntoTransmissionBuffer(CARRIAGE_RETURN);
				break;
			case '4':
				FAT32_Test(5);
				UART0_PushByteIntoTransmissionBuffer(LINE_FEED);
				UART0_PushByteIntoTransmissionBuffer(CARRIAGE_RETURN);
				break;
			case '5':
				FAT32_Test(1);
				UART0_PushByteIntoTransmissionBuffer(LINE_FEED);
				UART0_PushByteIntoTransmissionBuffer(CARRIAGE_RETURN);
				break;
			case '6':
				FAT32_Test(0);
				UART0_PushByteIntoTransmissionBuffer(LINE_FEED);
				UART0_PushByteIntoTransmissionBuffer(CARRIAGE_RETURN);
				break;
			case '7':
				SDMMC_Test(5);
				UART0_PushByteIntoTransmissionBuffer(LINE_FEED);
				UART0_PushByteIntoTransmissionBuffer(CARRIAGE_RETURN);
				break;
			case '8':
				AY3891X_Test(1);
				UART0_PushByteIntoTransmissionBuffer(LINE_FEED);
				UART0_PushByteIntoTransmissionBuffer(CARRIAGE_RETURN);
				break;
			case '9':
				AY3891X_Test(2);
				UART0_PushByteIntoTransmissionBuffer(LINE_FEED);
				UART0_PushByteIntoTransmissionBuffer(CARRIAGE_RETURN);
				break;
			case 'q':
				AY3891X_Test(3);
				UART0_PushByteIntoTransmissionBuffer(LINE_FEED);
				UART0_PushByteIntoTransmissionBuffer(CARRIAGE_RETURN);
				break;				
			case 'w':
				AY3891X_Test(4);
				UART0_PushByteIntoTransmissionBuffer(LINE_FEED);
				UART0_PushByteIntoTransmissionBuffer(CARRIAGE_RETURN);
				break;				
			case 'e':
				AY3891X_Test(5);
				UART0_PushByteIntoTransmissionBuffer(LINE_FEED);
				UART0_PushByteIntoTransmissionBuffer(CARRIAGE_RETURN);
				break;
			case 'r':
				AY3891X_Test(6);
				UART0_PushByteIntoTransmissionBuffer(LINE_FEED);
				UART0_PushByteIntoTransmissionBuffer(CARRIAGE_RETURN);
				break;	
			case 't':
				AY3891X_Test(7);
				UART0_PushByteIntoTransmissionBuffer(LINE_FEED);
				UART0_PushByteIntoTransmissionBuffer(CARRIAGE_RETURN);	
				break;
			case 'y':
				AY3891X_Test(8);
				UART0_PushByteIntoTransmissionBuffer(LINE_FEED);
				UART0_PushByteIntoTransmissionBuffer(CARRIAGE_RETURN);	
				break;
			case 'u':
				AY3891X_Test(9);
				UART0_PushByteIntoTransmissionBuffer(LINE_FEED);
				UART0_PushByteIntoTransmissionBuffer(CARRIAGE_RETURN);	
				break;
			case 'i':
				AY3891X_Test('a');
				UART0_PushByteIntoTransmissionBuffer(LINE_FEED);
				UART0_PushByteIntoTransmissionBuffer(CARRIAGE_RETURN);	
				break;
			case 'o':
				AY3891X_Test('b');
				UART0_PushByteIntoTransmissionBuffer(LINE_FEED);
				UART0_PushByteIntoTransmissionBuffer(CARRIAGE_RETURN);	
				break;
			case 'p':
				AY3891X_Test('c');
				UART0_PushByteIntoTransmissionBuffer(LINE_FEED);
				UART0_PushByteIntoTransmissionBuffer(CARRIAGE_RETURN);	
				break;
			case 'a':
				AY3891X_Test('d');
				UART0_PushByteIntoTransmissionBuffer(LINE_FEED);
				UART0_PushByteIntoTransmissionBuffer(CARRIAGE_RETURN);	
				break;
			case 's':
				AY3891X_Test('e');
				UART0_PushByteIntoTransmissionBuffer(LINE_FEED);
				UART0_PushByteIntoTransmissionBuffer(CARRIAGE_RETURN);	
				break;
			case 'd':
				AY3891X_Test('f');
				UART0_PushByteIntoTransmissionBuffer(LINE_FEED);
				UART0_PushByteIntoTransmissionBuffer(CARRIAGE_RETURN);	
				break;
			case 'f':
				AY3891X_Test('g');
				UART0_PushByteIntoTransmissionBuffer(LINE_FEED);
				UART0_PushByteIntoTransmissionBuffer(CARRIAGE_RETURN);	
				break;
			case 'g':
				AY3891X_Test('h');
				UART0_PushByteIntoTransmissionBuffer(LINE_FEED);
				UART0_PushByteIntoTransmissionBuffer(CARRIAGE_RETURN);	
				break;
			case 'h':
				AY3891X_Test('i');
				UART0_PushByteIntoTransmissionBuffer(LINE_FEED);
				UART0_PushByteIntoTransmissionBuffer(CARRIAGE_RETURN);	
				break;
			case 'j':
				AY3891X_Test('j');
				UART0_PushByteIntoTransmissionBuffer(LINE_FEED);
				UART0_PushByteIntoTransmissionBuffer(CARRIAGE_RETURN);	
				break;

			default:
				UART0_PushByteIntoTransmissionBuffer('?');
				UART0_PushByteIntoTransmissionBuffer(LINE_FEED);
				UART0_PushByteIntoTransmissionBuffer(CARRIAGE_RETURN);
				break;
		};//switch
	
		UART0_PushByteIntoTransmissionBuffer('>');
		UART0_PushByteIntoTransmissionBuffer('>');
		UART0_PushByteIntoTransmissionBuffer(LINE_FEED);
		UART0_PushByteIntoTransmissionBuffer(CARRIAGE_RETURN);
	
	};//if

}//UART0_ParseRecivedData



/*	Description:
	 Main motor of the UARTPARSER module. 
	Recives:
	Returns:
	 By value:
	 By reference:
 */
void UART0_Motor(){
		// checks the transmission buffer and sends one byte if there are bytes waiting to be sent 
		UART0_PollBufferTransmission( );
		// reads bytes from the reception buffer and executes an action depending on read value
		UART0_ParseRecivedBytes();
};//UART0_Motor
